/*******************************************************************************
* File Name: Agnd.c  
* Version 1.20
*
*  Description:
*    This file provides the source code to the API for the 8-bit Voltage DAC 
*    (VDAC8) User Module.
*
*   Note:
*     Any unusual or non-standard behavior should be noted here. Other-
*     wise, this section should remain blank.
*
*******************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/




#include "cytypes.h"
#include "Agnd.h"

uint8 Agnd_initVar = 0;

/*******************************************************************************
* Function Name: Agnd_Start
********************************************************************************
*
* Summary:
*  The start function initializes the voltage DAC with the default values, 
*  and sets the power to the given level.  A power level of 0, is the same as executing
*  the stop function.
*
* Parameters:  
*  Power:   Sets power level between off (0) and (3) high power
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Agnd_Start(void) 
{
  
   /* Hardware initiazation only needs to occure the first time */
   if ( Agnd_initVar == 0)  
   {    
     Agnd_initVar = 1;
      Agnd_CR0 = (Agnd_MODE_V )  ;    

      /* Set default data source */
      if(Agnd_DEFAULT_DATA_SRC != 0 )    
      {
          Agnd_CR1 = (Agnd_DEFAULT_CNTL | Agnd_DACBUS_ENABLE) ;   
      }
      else
      {
          Agnd_CR1 = (Agnd_DEFAULT_CNTL | Agnd_DACBUS_DISABLE) ;  
      } 

       /* Set default strobe mode */
      if(Agnd_DEFAULT_STRB != 0)
      {
          Agnd_Strobe |= Agnd_STRB_EN ;
      }

		/* Set default range */
      Agnd_SetRange(Agnd_DEFAULT_RANGE); 
	  
      /* Set default speed */
      Agnd_SetSpeed(Agnd_DEFAULT_SPEED); 

      /* Enable power to DAC */
      Agnd_PWRMGR |= Agnd_ACT_PWR_EN;

      /* Set default value */
      Agnd_SetValue(Agnd_DEFAULT_DATA);

   }
   else
   {
       /* Enable power to DAC */
       Agnd_PWRMGR |= Agnd_ACT_PWR_EN;
   }

}


/*******************************************************************************
* Function Name: Agnd_Stop
********************************************************************************
*
* Summary:
*  Powers down DAC to lowest power state.
*
* Parameters:  
*  (void)  
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Agnd_Stop(void)
{
   /* Disble power to DAC */
   Agnd_PWRMGR &= ~Agnd_ACT_PWR_EN;
}


/*******************************************************************************
* Function Name: Agnd_SetSpeed
********************************************************************************
*
* Summary:
*  Set DAC speed
*
* Parameters:  
*  power:   Sets speed value 
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Agnd_SetSpeed(uint8 speed) 
{
   /* Clear power mask then write in new value */
   Agnd_CR0 &= ~Agnd_HS_MASK ;    
   Agnd_CR0 |=  ( speed & Agnd_HS_MASK) ;    
}


/*******************************************************************************
* Function Name: Agnd_SetRange
********************************************************************************
*
* Summary:
*  Set one of three current ranges.
*
* Parameters:  
*  Range:  Sets one of Three valid ranges.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Agnd_SetRange(uint8 range)
{
   Agnd_CR0 &= ~Agnd_RANGE_MASK ;      /* Clear existing mode */
   Agnd_CR0 |= ( range & Agnd_RANGE_MASK ) ; /*  Set Range  */
   Agnd_DacTrim();
}


/*******************************************************************************
* Function Name: Agnd_SetValue
********************************************************************************
*
* Summary:
*  Set 8-bit DAC value
*
* Parameters:  
*  value:  Sets DAC value between 0 and 255.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Agnd_SetValue(uint8 value)
{
   Agnd_Data = value;                             /*  Set Value  */
   
   #if defined(Agnd_FIRST_SILICON)
   Agnd_Data = value;                             /*  TODO: Remove when Silicon fix comes */
   #endif
}


/*******************************************************************************
* Function Name: Agnd_DacTrim
********************************************************************************
*
* Summary:
*  Set the trim value for the given range.
*
* Parameters:  
*   range:  1V or 4V range.  See constants.  
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Agnd_DacTrim(void)
{
    uint8 mode;
	
	mode = ((Agnd_CR0 & Agnd_RANGE_MASK) >> 2) + Agnd_TRIM_M7_1V_RNG_OFFSET;
	
	Agnd_TR = CY_GET_XTND_REG8((uint8 *)(Agnd_DAC_TRIM_BASE + mode));
}


/* [] END OF FILE */

