/*******************************************************************************
* File Name: CalDac.c  
* Version 1.10
*
*  Description:
*    This file provides the source code to the API for the 8-bit Current 
*    DAC (IDAC8) User Module.
*
*   Note:
*     
*
*******************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/



#include "cytypes.h"
#include "CalDac.h"   

uint8 CalDac_initVar = 0;

/*******************************************************************************
* Function Name: CalDac_Start
********************************************************************************
* Summary:
*  Set power level then turn on IDAC8.
*
* Parameters:  
*  power:   Sets power level between off (0) and (3) high power
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void CalDac_Start(void) 
{
   /* Hardware initiazation only needs to occure the first time */
   if ( CalDac_initVar == 0)  
   {     
      CalDac_initVar = 1;
      CalDac_CR0 = (CalDac_MODE_I | CalDac_DEFAULT_RANGE )  ;    

      /* Set default data source */
      if(CalDac_DEFAULT_DATA_SRC != 0 )    
      {
          CalDac_CR1 = (CalDac_DEFAULT_CNTL | CalDac_DACBUS_ENABLE | CalDac_DEFAULT_POLARITY) ;   
      }
      else
      {
          CalDac_CR1 = (CalDac_DEFAULT_CNTL | CalDac_DACBUS_DISABLE | CalDac_DEFAULT_POLARITY) ;   
      } 

       /* Set default strobe mode */
      if(CalDac_DEFAULT_STRB != 0)
      {
          CalDac_Strobe |= CalDac_STRB_EN ;
      }

      /* Set default speed */
      CalDac_SetSpeed(CalDac_DEFAULT_SPEED); 

      /* Enable power to DAC */
      CalDac_PWRMGR |= CalDac_ACT_PWR_EN;

      /* Set default value */
      CalDac_SetValue(CalDac_DEFAULT_DATA);
      
      /* Set proper DAC trim */
      CalDac_DacTrim();
   }
   else
   {
      /* Enable power to DAC */
      CalDac_PWRMGR |= CalDac_ACT_PWR_EN;
   }
}


/*******************************************************************************
* Function Name: CalDac_Stop
********************************************************************************
* Summary:
*  Powers down IDAC8 to lowest power state.
*
* Parameters:  
*   (void)
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void CalDac_Stop(void)
{
   /* Disble power to DAC */
   CalDac_PWRMGR &= ~CalDac_ACT_PWR_EN;
}

/*******************************************************************************
* Function Name: CalDac_SetSpeed
********************************************************************************
* Summary:
*  Set DAC speed
*
* Parameters:  
*  power:   Sets speed value
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void CalDac_SetSpeed(uint8 speed) 
{
   /* Clear power mask then write in new value */
   CalDac_CR0 &= ~CalDac_HS_MASK ;    
   CalDac_CR0 |=  ( speed & CalDac_HS_MASK) ;    
}

/*******************************************************************************
* Function Name: CalDac_SetPolarity
********************************************************************************
* Summary:
*  Sets IDAC to Sink or Source current
*
* Parameters:  
*  Polarity: Sets the IDAC to Sink or Source
*   0x00 - Source
*   0x04 - Sink
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void CalDac_SetPolarity(uint8 polarity)
{
   	CalDac_CR1 &= ~CalDac_IDIR_MASK;                /* clear polarity bit */
    CalDac_CR1 |= (polarity & CalDac_IDIR_MASK);    /* set new value */
    
    CalDac_DacTrim();
}

/*******************************************************************************
* Function Name: CalDac_SetRange
********************************************************************************
* Summary:
*  Set current range
*
* Parameters:  
*  Range:  Sets on of four valid ranges.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void CalDac_SetRange(uint8 range)
{
   CalDac_CR0 &= ~CalDac_RANGE_MASK ;       /* Clear existing mode */
   CalDac_CR0 |= ( range & CalDac_RANGE_MASK );  /*  Set Range  */
   CalDac_DacTrim();
}


/*******************************************************************************
* Function Name: CalDac_SetValue
********************************************************************************
* Summary:
*  Set DAC value
*
* Parameters:  
*  value:  Sets DAC value between 0 and 255.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void CalDac_SetValue(uint8 value)
{
   CalDac_Data = value;         /*  Set Value  */
   
   #if defined(CalDac_FIRST_SILICON)
   CalDac_Data = value;         /*  TODO: Need for first version of silicon */
   #endif
}

/*******************************************************************************
* Function Name: CalDac_DacTrim
********************************************************************************
* Summary:
*  Set the trim value for the given range.
*
* Parameters:  
*  None
*
* Return: 
*  (void) 
*
* Theory: 
*  Trim values for the VIDAC blocks are stored in the "Customer Table" area in 
*  Row 1 of the Hidden Flash.  There are 8 bytes of trim data for each VIDAC block.
*  The values are:
*       I Gain offset, min range, Sourcing
*       I Gain offset, min range, Sinking
*       I Gain offset, med range, Sourcing
*       I Gain offset, med range, Sinking
*       I Gain offset, max range, Sourcing
*       I Gain offset, max range, Sinking
*       V Gain offset, 1V range
*       V Gain offset, 4V range
*
* Side Effects:
*
*******************************************************************************/
void CalDac_DacTrim(void)
{
    uint8 mode;

    mode = ((CalDac_CR0 & CalDac_RANGE_MASK) >> 1);
    
    if((CalDac_IDIR_MASK & CalDac_CR1) == CalDac_IDIR_SINK)
    {
        mode++;
    }

    CalDac_TR = CY_GET_XTND_REG8((uint8 *)(CalDac_DAC_TRIM_BASE + mode));
}

/* [] END OF FILE */


