/*******************************************************************************
* File Name: Drive.c
* Version 1.90
*
* Description:
*  This file provides the source code to the API for OpAmp (Analog Buffer) 
*  Component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "Drive.h"

uint8 Drive_initVar = 0u;


/*******************************************************************************   
* Function Name: Drive_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the 
*  customizer of the component placed onto schematic. Usually called in 
*  Drive_Start().
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void Drive_Init(void) 
{
    Drive_SetPower(Drive_DEFAULT_POWER);
}


/*******************************************************************************   
* Function Name: Drive_Enable
********************************************************************************
*
* Summary:
*  Enables the OpAmp block operation
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void Drive_Enable(void) 
{
    /* Enable negative charge pumps in ANIF */
    Drive_PUMP_CR1_REG  |= (Drive_PUMP_CR1_CLKSEL | Drive_PUMP_CR1_FORCE);

    /* Enable power to buffer in active mode */
    Drive_PM_ACT_CFG_REG |= Drive_ACT_PWR_EN;

    /* Enable power to buffer in alternative active mode */
    Drive_PM_STBY_CFG_REG |= Drive_STBY_PWR_EN;
}


/*******************************************************************************
* Function Name:   Drive_Start
********************************************************************************
*
* Summary:
*  The start function initializes the Analog Buffer with the default values and 
*  sets the power to the given level. A power level of 0, is same as 
*  executing the stop function.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  Drive_initVar: Used to check the initial configuration, modified 
*  when this function is called for the first time.
*
*******************************************************************************/
void Drive_Start(void) 
{
    if(Drive_initVar == 0u)
    {
        Drive_initVar = 1u;
        Drive_Init();
    }

    Drive_Enable();
}


/*******************************************************************************
* Function Name: Drive_Stop
********************************************************************************
*
* Summary:
*  Powers down amplifier to lowest power state.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void Drive_Stop(void) 
{
    /* Disable power to buffer in active mode template */
    Drive_PM_ACT_CFG_REG &= (uint8)(~Drive_ACT_PWR_EN);

    /* Disable power to buffer in alternative active mode template */
    Drive_PM_STBY_CFG_REG &= (uint8)(~Drive_STBY_PWR_EN);
    
    /* Disable negative charge pumps for ANIF only if all ABuf is turned OFF */
    if(Drive_PM_ACT_CFG_REG == 0u)
    {
        Drive_PUMP_CR1_REG &= (uint8)(~(Drive_PUMP_CR1_CLKSEL | Drive_PUMP_CR1_FORCE));
    }
}


/*******************************************************************************
* Function Name: Drive_SetPower
********************************************************************************
*
* Summary:
*  Sets power level of Analog buffer.
*
* Parameters: 
*  power: PSoC3: Sets power level between low (1) and high power (3).
*         PSoC5: Sets power level High (0)
*
* Return:
*  void
*
**********************************************************************************/
void Drive_SetPower(uint8 power) 
{
    #if (CY_PSOC3 || CY_PSOC5LP)
        Drive_CR_REG &= (uint8)(~Drive_PWR_MASK);
        Drive_CR_REG |= power & Drive_PWR_MASK;      /* Set device power */
    #else
        CYASSERT(Drive_HIGHPOWER == power);
    #endif /* CY_PSOC3 || CY_PSOC5LP */
}


/* [] END OF FILE */
