
/*******************************************************************************
* File Name: Excite_Drive.c  
* Version 1.10
*
*  Description:
*    This file provides the source code to the API for the TIA User Module.
*
*******************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/



#include "cytypes.h"
#include "Excite_Drive.h"

#if(Excite_Drive_MIN_VDDA)
#include "Excite_Drive_bst_clk.h"
#endif

/* Fixed configuration of SC Block only performed once */
uint8 Excite_Drive_initVar = 0;

/*******************************************************************************
* Function Name: Excite_Drive_Start
********************************************************************************
* Summary:
*  The start function initializes the TIA with the default values and enables
*  power to the SC block
*
* Parameters:  
*  void
*
* Return: 
*  void 
*
* Theory: 
*  Full initialization of the SC Block configuration registers is only perfomed the
*  first time the routine executes - global variable Excite_Drive_initVar is used
*  indicate that the static configuration has been completed. 
*
* Side Effects:
*
*******************************************************************************/
void Excite_Drive_Start(void) 
{
    if(Excite_Drive_initVar == 0)
    {    
        Excite_Drive_initVar = 1;  

        /* Set SC Block Mode to TIA - SCxx_CR0[3:1] */
        Excite_Drive_CR0 = Excite_Drive_MODE_TIA;  
	
        /* 
            Set SC Block CR1 Control Register fields to support TIA mode 
                drive field set by SetPower() API
                compensation Cap - 5.1 pF
                div2 - disable, n/a for TIA mode
                Mixer/PGA Gain - 0db, n/a for TIA mode
        */
        Excite_Drive_CR1 = (Excite_Drive_COMP_5P1PF |
                                Excite_Drive_DIV2_DISABLE |
                                Excite_Drive_GAIN_0DB);  

        /* 
            Set SC Block CR2 Control Register fields to support TIA mode
                Bias - Low
                r20_r40 - 40K, n/a for TIA mode
                redc - set by SetCapFB() API
                rval - set by SetResFB() API
                PGA Ground Ref - disable, n/a for TIA mode
        */
        Excite_Drive_CR2 = (Excite_Drive_BIAS_LOW |
                                Excite_Drive_R20_40B_40K |
                                Excite_Drive_GNDVREF_DI);
                          
        /* Set default resistive feedback value */
        Excite_Drive_SetResFB(Excite_Drive_INIT_RES_FEEDBACK);
      
        /* Set default capacitive feedback value */
        Excite_Drive_SetCapFB(Excite_Drive_INIT_CAP_FEEDBACK);

        /* Set default power */
        Excite_Drive_SetPower(Excite_Drive_INIT_POWER);
    }

    /* Enable the Amp */
    Excite_Drive_PWRMGR |= Excite_Drive_ACT_PWR_EN;
   
    /* Enable SC Block boost clock control for low Vdda operation */
#if(Excite_Drive_MIN_VDDA)
    Excite_Drive_bst_clk_Enable();
    Excite_Drive_BSTCLK |= Excite_Drive_BST_CLK_EN;
#endif   
}

/*******************************************************************************
* Function Name: Excite_Drive_Stop
********************************************************************************
* Summary:
*  Disables power to SC block.
*
* Parameters:  
*  void 
*
* Return: 
*  void 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Excite_Drive_Stop(void)
{
    /* Disble power to the Amp */
    Excite_Drive_PWRMGR &= ~Excite_Drive_ACT_PWR_EN;
     
	 /* Disable SC Block boost clk control, if used (MinVdda < 2.7V) */
#if(Excite_Drive_MIN_VDDA)	
    Excite_Drive_BSTCLK &= ~Excite_Drive_BST_CLK_EN;
    Excite_Drive_bst_clk_Disable();
#endif 
}

/*******************************************************************************
* Function Name: Excite_Drive_SetPower
********************************************************************************
* Summary:
*  Set the drive power of the TIA
*
* Parameters:  
*  power:   Sets power level between (0) and (3) high power
*
* Return: 
*  void 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Excite_Drive_SetPower(uint8 power) 
{
    uint8 tmpCR;
    
    /* Sets drive bits in SC Block Control Register 1:  SCxx_CR[1:0] */    
    tmpCR = Excite_Drive_CR1 & ~Excite_Drive_DRIVE_MASK;
    tmpCR |= (power & Excite_Drive_DRIVE_MASK);
    Excite_Drive_CR1 = tmpCR;  
}

/*******************************************************************************
* Function Name: Excite_Drive_SetResFB
********************************************************************************
* Summary:
*  This function sets value for the resistive feedback of the amplifier.
*
* Parameters:  
*  res_feedback:  specifies resistive feedback value (see header file for gain values.)
*
* Return: 
*  void 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Excite_Drive_SetResFB(uint8 res_feedback)
{
   /* Only set new gain if it is a valid gain */
   if( res_feedback <= Excite_Drive_RES_FEEDBACK_MAX)
   {
      /* Clear SCxx_CR2 rval bits - SCxx_CR2[6:4]*/
      Excite_Drive_CR2 &= ~Excite_Drive_RVAL_MASK;

      /* Set resistive feedback value */
      Excite_Drive_CR2 |= (res_feedback << 4);
   }
}

/*******************************************************************************
* Function Name: Excite_Drive_SetCapFB
********************************************************************************
* Summary:
*  This function sets the value for the capacitive feedback of the amplifier.
*
* Parameters:  
*  cap_feedback:  specifies capacitive feedback value (see header file for gain values.)
*
* Return: 
*  void 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Excite_Drive_SetCapFB(uint8 cap_feedback)
{
   /* Only set new gain if it is a valid gain */
   if( cap_feedback <= Excite_Drive_CAP_FEEDBACK_MAX)
   {
      /* Clear SCxx_CR2 redc bits -  - CR2[3:2] */
      Excite_Drive_CR2 &= ~Excite_Drive_REDC_MASK;

      /* Set redc capacitive feedback value */
      Excite_Drive_CR2 |= (cap_feedback << 2);
   }
}


/* [] END OF FILE */
