/*******************************************************************************
* File Name: IDAC8_1.c  
* Version 1.10
*
*  Description:
*    This file provides the source code to the API for the 8-bit Current 
*    DAC (IDAC8) User Module.
*
*   Note:
*     
*
*******************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/



#include "cytypes.h"
#include "IDAC8_1.h"   

uint8 IDAC8_1_initVar = 0;

/*******************************************************************************
* Function Name: IDAC8_1_Start
********************************************************************************
* Summary:
*  Set power level then turn on IDAC8.
*
* Parameters:  
*  power:   Sets power level between off (0) and (3) high power
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_1_Start(void) 
{
   /* Hardware initiazation only needs to occure the first time */
   if ( IDAC8_1_initVar == 0)  
   {     
      IDAC8_1_initVar = 1;
      IDAC8_1_CR0 = (IDAC8_1_MODE_I | IDAC8_1_DEFAULT_RANGE )  ;    

      /* Set default data source */
      if(IDAC8_1_DEFAULT_DATA_SRC != 0 )    
      {
          IDAC8_1_CR1 = (IDAC8_1_DEFAULT_CNTL | IDAC8_1_DACBUS_ENABLE | IDAC8_1_DEFAULT_POLARITY) ;   
      }
      else
      {
          IDAC8_1_CR1 = (IDAC8_1_DEFAULT_CNTL | IDAC8_1_DACBUS_DISABLE | IDAC8_1_DEFAULT_POLARITY) ;   
      } 

       /* Set default strobe mode */
      if(IDAC8_1_DEFAULT_STRB != 0)
      {
          IDAC8_1_Strobe |= IDAC8_1_STRB_EN ;
      }

      /* Set default speed */
      IDAC8_1_SetSpeed(IDAC8_1_DEFAULT_SPEED); 

      /* Enable power to DAC */
      IDAC8_1_PWRMGR |= IDAC8_1_ACT_PWR_EN;

      /* Set default value */
      IDAC8_1_SetValue(IDAC8_1_DEFAULT_DATA);
      
      /* Set proper DAC trim */
      IDAC8_1_DacTrim();
   }
   else
   {
      /* Enable power to DAC */
      IDAC8_1_PWRMGR |= IDAC8_1_ACT_PWR_EN;
   }
}


/*******************************************************************************
* Function Name: IDAC8_1_Stop
********************************************************************************
* Summary:
*  Powers down IDAC8 to lowest power state.
*
* Parameters:  
*   (void)
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_1_Stop(void)
{
   /* Disble power to DAC */
   IDAC8_1_PWRMGR &= ~IDAC8_1_ACT_PWR_EN;
}

/*******************************************************************************
* Function Name: IDAC8_1_SetSpeed
********************************************************************************
* Summary:
*  Set DAC speed
*
* Parameters:  
*  power:   Sets speed value
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_1_SetSpeed(uint8 speed) 
{
   /* Clear power mask then write in new value */
   IDAC8_1_CR0 &= ~IDAC8_1_HS_MASK ;    
   IDAC8_1_CR0 |=  ( speed & IDAC8_1_HS_MASK) ;    
}

/*******************************************************************************
* Function Name: IDAC8_1_SetPolarity
********************************************************************************
* Summary:
*  Sets IDAC to Sink or Source current
*
* Parameters:  
*  Polarity: Sets the IDAC to Sink or Source
*   0x00 - Source
*   0x04 - Sink
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_1_SetPolarity(uint8 polarity)
{
   	IDAC8_1_CR1 &= ~IDAC8_1_IDIR_MASK;                /* clear polarity bit */
    IDAC8_1_CR1 |= (polarity & IDAC8_1_IDIR_MASK);    /* set new value */
    
    IDAC8_1_DacTrim();
}

/*******************************************************************************
* Function Name: IDAC8_1_SetRange
********************************************************************************
* Summary:
*  Set current range
*
* Parameters:  
*  Range:  Sets on of four valid ranges.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_1_SetRange(uint8 range)
{
   IDAC8_1_CR0 &= ~IDAC8_1_RANGE_MASK ;       /* Clear existing mode */
   IDAC8_1_CR0 |= ( range & IDAC8_1_RANGE_MASK );  /*  Set Range  */
   IDAC8_1_DacTrim();
}


/*******************************************************************************
* Function Name: IDAC8_1_SetValue
********************************************************************************
* Summary:
*  Set DAC value
*
* Parameters:  
*  value:  Sets DAC value between 0 and 255.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_1_SetValue(uint8 value)
{
   IDAC8_1_Data = value;         /*  Set Value  */
   
   #if defined(IDAC8_1_FIRST_SILICON)
   IDAC8_1_Data = value;         /*  TODO: Need for first version of silicon */
   #endif
}

/*******************************************************************************
* Function Name: IDAC8_1_DacTrim
********************************************************************************
* Summary:
*  Set the trim value for the given range.
*
* Parameters:  
*  None
*
* Return: 
*  (void) 
*
* Theory: 
*  Trim values for the VIDAC blocks are stored in the "Customer Table" area in 
*  Row 1 of the Hidden Flash.  There are 8 bytes of trim data for each VIDAC block.
*  The values are:
*       I Gain offset, min range, Sourcing
*       I Gain offset, min range, Sinking
*       I Gain offset, med range, Sourcing
*       I Gain offset, med range, Sinking
*       I Gain offset, max range, Sourcing
*       I Gain offset, max range, Sinking
*       V Gain offset, 1V range
*       V Gain offset, 4V range
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_1_DacTrim(void)
{
    uint8 mode;

    mode = ((IDAC8_1_CR0 & IDAC8_1_RANGE_MASK) >> 1);
    
    if((IDAC8_1_IDIR_MASK & IDAC8_1_CR1) == IDAC8_1_IDIR_SINK)
    {
        mode++;
    }

    IDAC8_1_TR = CY_GET_XTND_REG8((uint8 *)(IDAC8_1_DAC_TRIM_BASE + mode));
}

/* [] END OF FILE */


