/*******************************************************************************
* File Name: IExcite.c  
* Version 1.20
*
*  Description:
*    This file provides the source code to the API for the 8-bit Current 
*    DAC (IDAC8) User Module.
*
*   Note:
*     
*
*******************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/



#include "cytypes.h"
#include "IExcite.h"   

uint8 IExcite_initVar = 0;

/*******************************************************************************
* Function Name: IExcite_Start
********************************************************************************
* Summary:
*  Set power level then turn on IDAC8.
*
* Parameters:  
*  power:   Sets power level between off (0) and (3) high power
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IExcite_Start(void) 
{
   /* Hardware initiazation only needs to occure the first time */
   if ( IExcite_initVar == 0)  
   {     
      IExcite_initVar = 1;
      IExcite_CR0 = (IExcite_MODE_I | IExcite_DEFAULT_RANGE )  ;    

      /* Set default data source */
      if(IExcite_DEFAULT_DATA_SRC != 0 )    
      {
          IExcite_CR1 = (IExcite_DEFAULT_CNTL | IExcite_DACBUS_ENABLE | IExcite_DEFAULT_POLARITY) ;   
      }
      else
      {
          IExcite_CR1 = (IExcite_DEFAULT_CNTL | IExcite_DACBUS_DISABLE | IExcite_DEFAULT_POLARITY) ;   
      } 

       /* Set default strobe mode */
      if(IExcite_DEFAULT_STRB != 0)
      {
          IExcite_Strobe |= IExcite_STRB_EN ;
      }

      /* Set default speed */
      IExcite_SetSpeed(IExcite_DEFAULT_SPEED); 

      /* Enable power to DAC */
      IExcite_PWRMGR |= IExcite_ACT_PWR_EN;

      /* Set default value */
      IExcite_SetValue(IExcite_DEFAULT_DATA);
      
      /* Set proper DAC trim */
      IExcite_DacTrim();
   }
   else
   {
      /* Enable power to DAC */
      IExcite_PWRMGR |= IExcite_ACT_PWR_EN;
   }
}


/*******************************************************************************
* Function Name: IExcite_Stop
********************************************************************************
* Summary:
*  Powers down IDAC8 to lowest power state.
*
* Parameters:  
*   (void)
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IExcite_Stop(void)
{
   /* Disble power to DAC */
   IExcite_PWRMGR &= ~IExcite_ACT_PWR_EN;
}

/*******************************************************************************
* Function Name: IExcite_SetSpeed
********************************************************************************
* Summary:
*  Set DAC speed
*
* Parameters:  
*  power:   Sets speed value
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IExcite_SetSpeed(uint8 speed) 
{
   /* Clear power mask then write in new value */
   IExcite_CR0 &= ~IExcite_HS_MASK ;    
   IExcite_CR0 |=  ( speed & IExcite_HS_MASK) ;    
}

/*******************************************************************************
* Function Name: IExcite_SetPolarity
********************************************************************************
* Summary:
*  Sets IDAC to Sink or Source current
*
* Parameters:  
*  Polarity: Sets the IDAC to Sink or Source
*   0x00 - Source
*   0x04 - Sink
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IExcite_SetPolarity(uint8 polarity)
{
   	IExcite_CR1 &= ~IExcite_IDIR_MASK;                /* clear polarity bit */
    IExcite_CR1 |= (polarity & IExcite_IDIR_MASK);    /* set new value */
    
    IExcite_DacTrim();
}

/*******************************************************************************
* Function Name: IExcite_SetRange
********************************************************************************
* Summary:
*  Set current range
*
* Parameters:  
*  Range:  Sets on of four valid ranges.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IExcite_SetRange(uint8 range)
{
   IExcite_CR0 &= ~IExcite_RANGE_MASK ;       /* Clear existing mode */
   IExcite_CR0 |= ( range & IExcite_RANGE_MASK );  /*  Set Range  */
   IExcite_DacTrim();
}


/*******************************************************************************
* Function Name: IExcite_SetValue
********************************************************************************
* Summary:
*  Set DAC value
*
* Parameters:  
*  value:  Sets DAC value between 0 and 255.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IExcite_SetValue(uint8 value)
{
   IExcite_Data = value;         /*  Set Value  */
   
   #if defined(IExcite_FIRST_SILICON)
   IExcite_Data = value;         /*  TODO: Need for first version of silicon */
   #endif
}

/*******************************************************************************
* Function Name: IExcite_DacTrim
********************************************************************************
* Summary:
*  Set the trim value for the given range.
*
* Parameters:  
*  None
*
* Return: 
*  (void) 
*
* Theory: 
*  Trim values for the VIDAC blocks are stored in the "Customer Table" area in 
*  Row 1 of the Hidden Flash.  There are 8 bytes of trim data for each VIDAC block.
*  The values are:
*       I Gain offset, min range, Sourcing
*       I Gain offset, min range, Sinking
*       I Gain offset, med range, Sourcing
*       I Gain offset, med range, Sinking
*       I Gain offset, max range, Sourcing
*       I Gain offset, max range, Sinking
*       V Gain offset, 1V range
*       V Gain offset, 4V range
*
* Side Effects:
*
*******************************************************************************/
void IExcite_DacTrim(void)
{
    uint8 mode;

    mode = ((IExcite_CR0 & IExcite_RANGE_MASK) >> 1);
    
    if((IExcite_IDIR_MASK & IExcite_CR1) == IExcite_IDIR_SINK)
    {
        mode++;
    }

    IExcite_TR = CY_GET_XTND_REG8((uint8 *)(IExcite_DAC_TRIM_BASE + mode));
}

/* [] END OF FILE */


