/*******************************************************************************
* File Name: Mixer_Buffer.c  
* Version 1.70
*
* Description:
*  This file provides the source code to the API for the PGA 
*  User Module.
*
* Note:
*
********************************************************************************
* Copyright 2008-2011, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "Mixer_Buffer.h"

#if(Mixer_Buffer_CYDEV_VDDA_MV < Mixer_Buffer_MINIMUM_VDDA_THRESHOLD_MV)
    #include "Mixer_Buffer_ScBoostClk.h"
#endif /* Mixer_Buffer_ACLK_ENABLE */

/* Check to see if required defines such as CY_PSOC3 and CY_PSOC5 are available */
/* They are defined starting with cy_boot v2.30 */
#ifndef CY_PSOC3
#error Component PGA_v1_70 requires cy_boot v2.30 or later
#endif /* CY_PSOC3 */

/* Constant array for gain settings */
const uint8 Mixer_Buffer_GainArray[9] = { 
   (Mixer_Buffer_RVAL_0K   | Mixer_Buffer_R20_40B_40K | Mixer_Buffer_BIAS_LOW), /* G=1  */
   (Mixer_Buffer_RVAL_40K  | Mixer_Buffer_R20_40B_40K | Mixer_Buffer_BIAS_LOW), /* G=2  */
   (Mixer_Buffer_RVAL_120K | Mixer_Buffer_R20_40B_40K | Mixer_Buffer_BIAS_LOW), /* G=4  */
   (Mixer_Buffer_RVAL_280K | Mixer_Buffer_R20_40B_40K | Mixer_Buffer_BIAS_LOW), /* G=8  */
   (Mixer_Buffer_RVAL_600K | Mixer_Buffer_R20_40B_40K | Mixer_Buffer_BIAS_LOW), /* G=16 */
   (Mixer_Buffer_RVAL_880K | Mixer_Buffer_R20_40B_40K | Mixer_Buffer_BIAS_LOW), /* G=24 */
   (Mixer_Buffer_RVAL_620K | Mixer_Buffer_R20_40B_20K | Mixer_Buffer_BIAS_LOW), /* G=32 */
   (Mixer_Buffer_RVAL_470K | Mixer_Buffer_R20_40B_20K | Mixer_Buffer_BIAS_LOW), /* G=48 */
   (Mixer_Buffer_RVAL_490K | Mixer_Buffer_R20_40B_20K | Mixer_Buffer_BIAS_LOW)  /* G=50 */
};

/* Constant array for gain compenstion settings */
const uint8 Mixer_Buffer_GainComp[9] = { 
   ( Mixer_Buffer_COMP_4P35PF  | ( Mixer_Buffer_REDC_00 >> 2 )), /* G=1  */
   ( Mixer_Buffer_COMP_4P35PF  | ( Mixer_Buffer_REDC_01 >> 2 )), /* G=2  */
   ( Mixer_Buffer_COMP_3P0PF   | ( Mixer_Buffer_REDC_01 >> 2 )), /* G=4  */
   ( Mixer_Buffer_COMP_3P0PF   | ( Mixer_Buffer_REDC_01 >> 2 )), /* G=8  */
   ( Mixer_Buffer_COMP_3P6PF   | ( Mixer_Buffer_REDC_01 >> 2 )), /* G=16 */
   ( Mixer_Buffer_COMP_3P6PF   | ( Mixer_Buffer_REDC_11 >> 2 )), /* G=24 */
   ( Mixer_Buffer_COMP_3P6PF   | ( Mixer_Buffer_REDC_11 >> 2 )), /* G=32 */
   ( Mixer_Buffer_COMP_3P6PF   | ( Mixer_Buffer_REDC_00 >> 2 )), /* G=48 */
   ( Mixer_Buffer_COMP_3P6PF   | ( Mixer_Buffer_REDC_00 >> 2 ))  /* G=50 */
};

uint8 Mixer_Buffer_initVar = 0u;

#if (CY_PSOC5_ES1)
uint8 Mixer_Buffer_restoreVal = 0u;
#endif /* CY_PSOC5_ES1 */

static Mixer_Buffer_BACKUP_STRUCT  Mixer_Buffer_backup;


/*******************************************************************************   
* Function Name: Mixer_Buffer_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the 
*  customizer of the component placed onto schematic. Usually called in 
*  Mixer_Buffer_Start().
*
* Parameters:
*  void
*
* Return:
*  void
*
* Reentrant:
*  No
* 
*******************************************************************************/
void Mixer_Buffer_Init(void) 
{
    /* Set PGA mode */
    Mixer_Buffer_CR0_REG = Mixer_Buffer_MODE_PGA;  
    
    /* Set non-inverting PGA mode  and reference mode */
    Mixer_Buffer_CR1_REG |= Mixer_Buffer_PGA_NINV;  

    /* Set default gain and ref mode */
    Mixer_Buffer_CR2_REG = Mixer_Buffer_VREF_MODE;

    /* Set gain and compensation */
    Mixer_Buffer_SetGain(Mixer_Buffer_DEFAULT_GAIN);

    /* Set power */
    Mixer_Buffer_SetPower(Mixer_Buffer_DEFAULT_POWER);
}


/*******************************************************************************   
* Function Name: Mixer_Buffer_Enable
********************************************************************************
*
* Summary:
*  Enables the PGA block operation
*
* Parameters:
*  void
*
* Return:
*  void
*
* Reentrant:
*  No
* 
*******************************************************************************/
void Mixer_Buffer_Enable(void) 
{
    /* If a boost clock is required  */
    #if(Mixer_Buffer_CYDEV_VDDA_MV < Mixer_Buffer_MINIMUM_VDDA_THRESHOLD_MV)
        Mixer_Buffer_ACLK_PM_ACT_CFG_REG |= Mixer_Buffer_ACLK_ACT_PWR_EN;
        Mixer_Buffer_ACLK_PM_STBY_CFG_REG |= Mixer_Buffer_ACLK_STBY_PWR_EN;
         Mixer_Buffer_ScBoostClk_Enable();  /* turn on boost pump */
        Mixer_Buffer_BST_REG |= 0x08;
    #endif /* Mixer_Buffer_ACLK_ENABLE */
   
    /* PSoC3 ES2 or early, PSoC5 ES1 */
    #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
        /* Enable Pump only if voltage is below threshold */
        if (Mixer_Buffer_CYDEV_VDDA_MV < Mixer_Buffer_MINIMUM_VDDA_THRESHOLD_MV)
        {
            Mixer_Buffer_SC_MISC_REG |= Mixer_Buffer_PUMP_FORCE;
        }
        
    /* PSoC3 ES3 or later, PSoC5 ES2 or later*/
    #elif (CY_PSOC3_ES3 || CY_PSOC5_ES2)
        /* Enable charge Pump clock for SC block */
        Mixer_Buffer_PUMP_CR1_REG |= Mixer_Buffer_PUMP_CR1_SC_CLKSEL;
        /* Enable Pump only if voltage is below threshold */
        if (Mixer_Buffer_CYDEV_VDDA_MV < Mixer_Buffer_MINIMUM_VDDA_THRESHOLD_MV)
        {
            Mixer_Buffer_SC_MISC_REG |= Mixer_Buffer_PUMP_FORCE;
        }
    #endif /* Mixer_Buffer_PSOC3_ES2 || Mixer_Buffer_PSOC5_ES1 */

    #if (CY_PSOC5_ES1)
    if(Mixer_Buffer_restoreVal == 1u)
    {
        Mixer_Buffer_CR1_REG = Mixer_Buffer_backup.scCr1Reg;
        Mixer_Buffer_CR2_REG =Mixer_Buffer_backup.scCr2Reg;
        Mixer_Buffer_restoreVal = 0u;
    }
    #endif /* CY_PSOC5_ES1 */
     
    /* Enable power to the Amp in Active mode*/
    Mixer_Buffer_PM_ACT_CFG_REG |= Mixer_Buffer_ACT_PWR_EN;

    /* Enable power to the Amp in Alternative Active mode*/
    Mixer_Buffer_PM_STBY_CFG_REG |= Mixer_Buffer_STBY_PWR_EN;
}


/*******************************************************************************
* Function Name: Mixer_Buffer_Start
********************************************************************************
*
* Summary:
*  The start function initializes the PGA with the default values, and sets
*  the power to the given level.  A power level of 0, is the same as executing
*  the stop function.
*
* Parameters:  
*  void
*
* Return: 
*  void 
*
* Reentrant: 
*  No
*
*******************************************************************************/
void Mixer_Buffer_Start(void) 
{

    /* This is to restore the value of register CR1 and CR2 which is saved 
      in prior to the modification in stop() API */
    

    if(Mixer_Buffer_initVar == 0u)
    {
        Mixer_Buffer_Init();
        Mixer_Buffer_initVar = 1u;
    }
    
    
    Mixer_Buffer_Enable();
}


/*******************************************************************************
* Function Name: Mixer_Buffer_Stop
********************************************************************************
*
* Summary:
*  Powers down amplifier to lowest power state.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant: 
*  No
*
*******************************************************************************/
void Mixer_Buffer_Stop(void) 
{
    /* Disable pumps only if only one SC block is in use */
    if (Mixer_Buffer_PM_ACT_CFG_REG == Mixer_Buffer_ACT_PWR_EN)
    {
       Mixer_Buffer_SC_MISC_REG &= ~Mixer_Buffer_PUMP_FORCE;
    }   
    /* Disble power to the Amp in Active mode template */
    Mixer_Buffer_PM_ACT_CFG_REG &= ~Mixer_Buffer_ACT_PWR_EN;

    /* Disble power to the Amp in Alternative Active mode template */
    Mixer_Buffer_PM_STBY_CFG_REG &= ~Mixer_Buffer_STBY_PWR_EN;

    /* If a boost clock is required  */
    /* Disable SC Block boost clk control, if used (MinVdda < 2.7V) */
    #if(Mixer_Buffer_CYDEV_VDDA_MV < Mixer_Buffer_MINIMUM_VDDA_THRESHOLD_MV)
        Mixer_Buffer_ACLK_PM_ACT_CFG_REG &= ~Mixer_Buffer_ACLK_ACT_PWR_EN;
        Mixer_Buffer_ACLK_PM_STBY_CFG_REG &= ~Mixer_Buffer_ACLK_STBY_PWR_EN;
        Mixer_Buffer_ScBoostClk_Disable();     
    #endif /* Mixer_Buffer_ACLK_ENABLE */

    /* This sets PGA in zero current mode and output routes are valid */
    #if (CY_PSOC5_ES1)
        Mixer_Buffer_backup.scCr1Reg = Mixer_Buffer_CR1_REG;
        Mixer_Buffer_backup.scCr2Reg = Mixer_Buffer_CR2_REG;
        Mixer_Buffer_CR1_REG = Mixer_Buffer_SC_REG_CLR;
        Mixer_Buffer_CR2_REG = Mixer_Buffer_SC_REG_CLR;
        Mixer_Buffer_restoreVal = 1u;
    #endif /* CY_PSOC5_ES1 */


}


/*******************************************************************************
* Function Name: Mixer_Buffer_SetPower
********************************************************************************
*
* Summary:
*  Set the power of the PGA
*
* Parameters:  
*  power:  Sets power level between (0) and (3) high power
*
* Return: 
*  void
*
* Reentrant: 
*  Yes
*
*******************************************************************************/
void Mixer_Buffer_SetPower(uint8 power) 
{
    uint8 tmpCR;

    tmpCR = Mixer_Buffer_CR1_REG & ~Mixer_Buffer_DRIVE_MASK;
    tmpCR |= (power & Mixer_Buffer_DRIVE_MASK);
    Mixer_Buffer_CR1_REG = tmpCR;  
}


/*******************************************************************************
* Function Name: Mixer_Buffer_SetGain
********************************************************************************
*
* Summary:
*  This function sets values of the input and feedback resistors to set a 
*  specific gain of the amplifier.
*
* Parameters:  
*  gain:  Gain value of PGA (See header file for gain values.)
*
* Return: 
*  void 
*
* Reentrant: 
*  Yes
*
*******************************************************************************/
void Mixer_Buffer_SetGain(uint8 gain) 
{
    /* Only set new gain if it is a valid gain */
    if( gain <= Mixer_Buffer_GAIN_MAX)
    {
        /* Clear resistors, redc, and bias */
        Mixer_Buffer_CR2_REG &= ~(Mixer_Buffer_RVAL_MASK | Mixer_Buffer_R20_40B_MASK | 
                                Mixer_Buffer_REDC_MASK | Mixer_Buffer_BIAS_MASK );

        /* Set gain value resistors, redc comp, and bias */
        Mixer_Buffer_CR2_REG |= (Mixer_Buffer_GainArray[gain] |
                                ( (Mixer_Buffer_GainComp[gain] << 2 ) & Mixer_Buffer_REDC_MASK));
             
        /* Clear sc_comp  */
        Mixer_Buffer_CR1_REG &= ~Mixer_Buffer_COMP_MASK;
        /* Set sc_comp  */
        Mixer_Buffer_CR1_REG |= ( Mixer_Buffer_GainComp[gain] | Mixer_Buffer_COMP_MASK );
    }
}


/* [] END OF FILE */
