
/*******************************************************************************
* File Name: TIA_Buffer.c  
* Version 1.20
*
*  Description:
*    This file provides the source code to the API for the PGA 
*    User Module.
*
*   Note:
*
********************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/




#include "cytypes.h"
#include "TIA_Buffer.h"

#if (TIA_Buffer_ACLK_ENABLE)
    #include "TIA_Buffer_ScBoostClk.h"
#endif


/* Constant array for gain settings */
const uint8 TIA_Buffer_GainArray[10] = { 
   (TIA_Buffer_RVAL_20K   | TIA_Buffer_R20_40B_40K | TIA_Buffer_BIAS_LOW), /* G=1  */
   (TIA_Buffer_RVAL_30K   | TIA_Buffer_R20_40B_40K | TIA_Buffer_BIAS_LOW), /* G=2  */
   (TIA_Buffer_RVAL_40K   | TIA_Buffer_R20_40B_40K | TIA_Buffer_BIAS_LOW), /* G=4  */
   (TIA_Buffer_RVAL_60K   | TIA_Buffer_R20_40B_40K | TIA_Buffer_BIAS_LOW), /* G=8  */
   (TIA_Buffer_RVAL_120K  | TIA_Buffer_R20_40B_40K | TIA_Buffer_BIAS_LOW), /* G=16 */
   (TIA_Buffer_RVAL_250K  | TIA_Buffer_R20_40B_40K | TIA_Buffer_BIAS_LOW), /* G=24 */
   (TIA_Buffer_RVAL_1000K | TIA_Buffer_R20_40B_40K | TIA_Buffer_BIAS_LOW), /* G=25 */
   (TIA_Buffer_RVAL_250K  | TIA_Buffer_R20_40B_20K | TIA_Buffer_BIAS_LOW), /* G=32 */
   (TIA_Buffer_RVAL_500K  | TIA_Buffer_R20_40B_20K | TIA_Buffer_BIAS_LOW), /* G=48 */
   (TIA_Buffer_RVAL_1000K | TIA_Buffer_R20_40B_20K | TIA_Buffer_BIAS_LOW)  /* G=50 */
};

/* Constant array for gain compenstion settings */
const uint8 TIA_Buffer_GainComp[10] = { 
   ( TIA_Buffer_COMP_4P35PF  | ( TIA_Buffer_REDC_00 >> 2 )), /* G=1  */
   ( TIA_Buffer_COMP_4P35PF  | ( TIA_Buffer_REDC_01 >> 2 )), /* G=2  */
   ( TIA_Buffer_COMP_3P0PF   | ( TIA_Buffer_REDC_01 >> 2 )), /* G=4  */
   ( TIA_Buffer_COMP_3P0PF   | ( TIA_Buffer_REDC_01 >> 2 )), /* G=8  */
   ( TIA_Buffer_COMP_3P6PF   | ( TIA_Buffer_REDC_01 >> 2 )), /* G=16 */
   ( TIA_Buffer_COMP_3P6PF   | ( TIA_Buffer_REDC_11 >> 2 )), /* G=24 */
   ( TIA_Buffer_COMP_3P0PF   | ( TIA_Buffer_REDC_10 >> 2 )), /* G=25 */
   ( TIA_Buffer_COMP_3P6PF   | ( TIA_Buffer_REDC_11 >> 2 )), /* G=32 */
   ( TIA_Buffer_COMP_3P6PF   | ( TIA_Buffer_REDC_00 >> 2 )), /* G=48 */
   ( TIA_Buffer_COMP_3P6PF   | ( TIA_Buffer_REDC_00 >> 2 ))  /* G=50 */
};

uint8 TIA_Buffer_initVar = 0;

/*******************************************************************************
* Function Name: TIA_Buffer_Start
********************************************************************************
* Summary:
*  The start function initializes the PGA with the default values, and sets
*  the power to the given level.  A power level of 0, is the same as executing
*  the stop function.
*
* Parameters:  
*  power:   Sets power level between off (0) and (3) high power
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void TIA_Buffer_Start(void) 
{
    if(TIA_Buffer_initVar == 0)
   {
      TIA_Buffer_initVar = 1;  

      /* Set PGA mode */
      TIA_Buffer_CR0 = TIA_Buffer_MODE_PGA;  
	
      /* Set non-inverting PGA mode  and reference mode */
      TIA_Buffer_CR1 |= TIA_Buffer_PGA_NINV;  

      /* Set default gain and ref mode */
      TIA_Buffer_CR2 = TIA_Buffer_VREF_MODE;

      /* Set gain and compensation */
      TIA_Buffer_SetGain(TIA_Buffer_DEFAULT_GAIN);

      /* Set power */
     TIA_Buffer_SetPower(TIA_Buffer_DEFAULT_POWER);
   }

   /* If a boost clock is required  */
   #if (TIA_Buffer_ACLK_ENABLE)
      TIA_Buffer_ACLK_PWRMGR |= TIA_Buffer_ACLK_ACT_PWR_EN;
      TIA_Buffer_ScBoostClk_Enable();  // turn on boost pump
	   TIA_Buffer_BST |= 0x08;
   #endif 

   /* Enable power to the Amp */
   TIA_Buffer_PWRMGR |= TIA_Buffer_ACT_PWR_EN;

}

/*******************************************************************************
* Function Name: TIA_Buffer_Stop
********************************************************************************
* Summary:
*  Powers down amplifier to lowest power state.
*
* Parameters:  
*  (void) 
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void TIA_Buffer_Stop(void)
{
   /* Disble power to the Amp */
   TIA_Buffer_PWRMGR &= ~TIA_Buffer_ACT_PWR_EN;

   /* If a boost clock is required  */
   /* Disable and power down boost clock */
   #if (TIA_Buffer_ACLK_ENABLE)
      TIA_Buffer_ACLK_PWRMGR &= ~TIA_Buffer_ACLK_ACT_PWR_EN;
      TIA_Buffer_ScBoostClk_Disable();  
   #endif 

}

/*******************************************************************************
* Function Name: TIA_Buffer_SetPower
********************************************************************************
* Summary:
*  Set the power of the PGA
*
* Parameters:  
*  power:   Sets power level between (0) and (3) high power
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void TIA_Buffer_SetPower(uint8 power) 
{
   uint8 tmpCR;

   tmpCR = TIA_Buffer_CR1 & ~TIA_Buffer_DRIVE_MASK;
   tmpCR |= (power & TIA_Buffer_DRIVE_MASK);
   TIA_Buffer_CR1 = tmpCR;  
}

/*******************************************************************************
* Function Name: TIA_Buffer_SetGain
********************************************************************************
* Summary:
*  This function sets values of the input and feedback resistors to set a 
*  specific gain of the amplifier.
*
* Parameters:  
*  gain:  Gain value of PGA (See header file for gain values.)
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void TIA_Buffer_SetGain(uint8 gain)
{
   /* Only set new gain if it is a valid gain */
   if( gain <= TIA_Buffer_GAIN_MAX)
   {
      /* Clear resistors, redc, and bias */
      TIA_Buffer_CR2 &= ~(TIA_Buffer_RVAL_MASK | TIA_Buffer_R20_40B_MASK | 
                                TIA_Buffer_REDC_MASK | TIA_Buffer_BIAS_MASK );

      /* Set gain value resistors, redc comp, and bias */
      TIA_Buffer_CR2 |= (TIA_Buffer_GainArray[gain] |
                             ( (TIA_Buffer_GainComp[gain] << 2 ) & TIA_Buffer_REDC_MASK) );
            
      /* Clear sc_comp  */
      TIA_Buffer_CR1 &= ~TIA_Buffer_COMP_MASK;
      /* Set sc_comp  */
      TIA_Buffer_CR1 |= ( TIA_Buffer_GainComp[gain] | TIA_Buffer_COMP_MASK );
   }
}


/* [] END OF FILE */
