
/*******************************************************************************
* File Name: Mixer.c  
* Version 1.10
*
*  Description:
*    This file provides the source code to the API for the MIXER User Module.
*
*******************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/



#include "cytypes.h"
#include "Mixer.h"

#if(Mixer_MIN_VDDA)
#include "Mixer_bst_clk.h"
#endif

/* Fixed configuration of SC Block only performed once */
uint8 Mixer_initVar = 0;

/*******************************************************************************
* Function Name: Mixer_Start
********************************************************************************
* Summary:
*  The start function initializes the SC block control registers based on the
*  Mixer type selected and and enables power to the SC block
*
* Parameters:  
*  void
*
* Return: 
*  void 
*
* Theory: 
*  Full initialization of the SC Block configuration registers is only perfomed the
*  first time the routine executes - global variable Mixer_initVar is used
*  indicate that the static configuration has been completed. 
*
* Side Effects:
*
*******************************************************************************/
void Mixer_Start(void) 
{
    uint8   tempCR2 = 0;

    if (Mixer_initVar == 0)
    {    
        Mixer_initVar = 1;  

        /* Configure SC Block based on selected Mixer type */
        if (Mixer_MIXER_TYPE == Mixer_CTMIXER)    /* Continuous Time (up) mixer */
        {
            /* SC Block mode -  - SCxx_CR0[3:1] */
            Mixer_CR0 = Mixer_MODE_CTMIXER;   

            /* SC Block CR1 */ 
            Mixer_CR1 = (Mixer_COMP_4P35PF  |
                                    Mixer_DIV2_DISABLE |
                                    Mixer_GAIN_0DB);                                 

            /* SC Block CR2 */
            tempCR2 = (Mixer_BIAS_LOW |
                       Mixer_REDC_01 |
                       Mixer_GNDVREF_DI);

            /* 
                Set SC Block resistor values based on Local Oscillator being above or below 100kHz
                - controled by r20_40 (Rmix) bit for CT mode mixer 
            */
            if (Mixer_LO_FREQ_FLAG == Mixer_LO_LT_100K)
            {
                tempCR2 |= Mixer_R20_40B_40K; 
            }
            else
            {
                tempCR2 |= Mixer_R20_40B_20K;
            }
            Mixer_CR2 = tempCR2;
        }
        else
        {    /* Discrete Time (down) mixer */
            /* SC Block mode -  - SCxx_CR0[3:1] */
            Mixer_CR0 = Mixer_MODE_DTMIXER;       

            /* SC Block CR1 */ 
            Mixer_CR1 = (Mixer_COMP_4P35PF  |
                                    Mixer_DIV2_ENABLE |
                                    Mixer_GAIN_0DB);                                 

            /* SC Block CR2 */
            tempCR2 = (Mixer_BIAS_LOW |Mixer_GNDVREF_DI);

            /* 
                Set SC Block resistor values based on Local Oscillator beign above or below 100kHz
                - set r20_40 (input resitance) and rval (feedback) resistor values seperately for DT mode
            */
            if (Mixer_LO_FREQ_FLAG == Mixer_LO_LT_100K)
            {
                tempCR2 |= (Mixer_R20_40B_40K | Mixer_RVAL_40K); 
            }
            else
            {
                tempCR2 |= (Mixer_R20_40B_20K | Mixer_RVAL_20K);
            }
            Mixer_CR2 = tempCR2; 
        } /* end if - Mixer Type */

        /* Enable SC Block clocking */
        Mixer_CLK |= Mixer_CLK_EN;

        /* Set default power */
        Mixer_SetPower(Mixer_INIT_POWER);
    }

    /* Enable SC block */
    Mixer_PWRMGR |= Mixer_ACT_PWR_EN;

    /* Enable SC Block boost clock control for low Vdda operation */
#if(Mixer_MIN_VDDA)
    Mixer_BSTCLK |= Mixer_BST_CLK_EN;     /* enable for Vdda < 2.7V */
    Mixer_bst_clk_Enable();
#endif
}


/*******************************************************************************
* Function Name: Mixer_Stop
********************************************************************************
* Summary:
*  Disables power to SC block.
*
* Parameters:  
*  void 
*
* Return: 
*  void 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Mixer_Stop(void)
{
    /* Disble power to the Amp */
    Mixer_PWRMGR &= ~Mixer_ACT_PWR_EN;

    /* Disable SC Block boost clk control, if used (MinVdda < 2.7V) */
#if(Mixer_MIN_VDDA)	
    Mixer_BSTCLK &= ~Mixer_BST_CLK_EN;
    Mixer_bst_clk_Disable();
#endif    
}


/*******************************************************************************
* Function Name: Mixer_SetPower
********************************************************************************
* Summary:
*  Set the drive power of the MIXER
*
* Parameters:  
*  power:   Sets power level between (0) and (3) high power
*
* Return: 
*  void 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void Mixer_SetPower(uint8 power) 
{
    uint8 tmpCR;

    /* Sets drive bits in SC Block Control Register 1:  SCxx_CR[1:0] */    
    tmpCR = Mixer_CR1 & ~Mixer_DRIVE_MASK;
    tmpCR |= (power & Mixer_DRIVE_MASK);
    Mixer_CR1 = tmpCR;  
}


/* [] END OF FILE */
